// 🎯 Dart imports:
import 'dart:typed_data';

// 🐦 Flutter imports:
import 'package:flutter/foundation.dart';
import 'package:flutter/material.dart';
import 'package:shared_preferences/shared_preferences.dart';

// 📦 Package imports:
import 'package:verttice_types/verttice_types.dart';
import 'package:verttice_ui/src/models/themed_locale/themed_locale_index.dart';
import 'package:verttice_ui/src/widgets/cookies_consent.dart';
import 'package:url_strategy/url_strategy.dart';
import 'package:verttice_ui/verttice_ui.dart';

class ThemedColorsConfig {
  final ThemedColor themedColors;
  final bool forceInitColor;

  ThemedColorsConfig({
    required this.themedColors,
    this.forceInitColor = false,
  });
}

class ExpandableOptionsConfig {
  final Future<List<String>> Function({
    required String optionsID,
  }) getOptions;

  final Future<void> Function({
    required String optionsID,
    required String option,
  }) setOption;

  final Future<void> Function({
    required String optionsID,
    required String option,
  }) deleteOption;

  ExpandableOptionsConfig({
    required this.getOptions,
    required this.setOption,
    required this.deleteOption,
  });
}

class VertticeUI {
  static const double appBarHeight = 50;
  static const double commonDivider = 2.5;
  static const double commonDesktopSideSelectMenu = 230;
  VertticeUI._privateConstructor();
  static final VertticeUI instance = VertticeUI._privateConstructor();
  final ThemedLocaleManager _localeManager = ThemedLocaleManager();
  ThemedColorsConfig _themedColors = ThemedColorsConfig(
    themedColors: ThemedColor.red,
  );

  CookiesConsent _cookiesConsent = CookiesConsent();

  late ExpandableOptionsConfig _expandableOptionsConfig;

  late Future<List<FileModel>> Function({
    required BuildContext context,
    required List<UploadFileDataModel> files,
  }) _uploadFiles;

  late Future<String> Function({
    required String path,
  }) _getStorageDownloadURL;

  late Future<Uint8List?> Function({
    required String path,
  }) _getStorageData;

  late String _firebaseApiKey;

  ExpandableOptionsConfig get expandableOptionsConfig =>
      _expandableOptionsConfig;

  CookiesConsent get cookiesConsent => _cookiesConsent;
  ThemedLocaleManager get localeManager => _localeManager;
  ThemedColorsConfig get themedColors => _themedColors;

  Future<void> ensureInitialized() async {
    if (kIsWeb) {
      setPathUrlStrategy();
    }
    await _initCookiesConsent();
  }

  void initThemedColors(ThemedColorsConfig color) {
    _themedColors = color;
  }

  Future<void> setCookieConsent(CookiesConsent cookiesConsent) async {
    _cookiesConsent = cookiesConsent;
    final pref = await SharedPreferences.getInstance();
    await pref.setBool(
      "alreadyRequestCookiesConsent",
      _cookiesConsent.alreadyRequestCookiesConsent,
    );
    await pref.setBool("trackingCookies", _cookiesConsent.tracking.allow);
    await pref.setBool("analyticsCookies", _cookiesConsent.analytics.allow);
    await pref.setBool("marketingCookies", _cookiesConsent.marketing.allow);
  }

  Future<List<FileModel>> Function({
    required BuildContext context,
    required List<UploadFileDataModel> files,
  }) get uploadFiles => _uploadFiles;

  Future<String?> Function({
    required String path,
  }) get getStorageDownloadURL => _getStorageDownloadURL;

  Future<Uint8List?> Function({
    required String path,
  }) get getStorageData => _getStorageData;

  String get firebaseApiKey => _firebaseApiKey;

  Future<void> _initCookiesConsent() async {
    final pref = await SharedPreferences.getInstance();
    final alreadyRequestCookiesConsent = pref.getBool(
          "alreadyRequestCookiesConsent",
        ) ??
        false;
    final tracking = pref.getBool("trackingCookies") ?? false;
    final analytics = pref.getBool("analyticsCookies") ?? false;
    final marketing = pref.getBool("marketingCookies") ?? false;
    _cookiesConsent = _cookiesConsent.copyWith(
      alreadyRequestCookiesConsent: alreadyRequestCookiesConsent,
      analytics: _cookiesConsent.analytics.copyWith(
        allow: analytics,
      ),
      marketing: _cookiesConsent.marketing.copyWith(
        allow: marketing,
      ),
      tracking: _cookiesConsent.tracking.copyWith(
        allow: tracking,
      ),
    );
  }

  void setExpandableOptionsConfig(
    ExpandableOptionsConfig expandableOptionsConfig,
  ) {
    _expandableOptionsConfig = expandableOptionsConfig;
  }

  void setUploadFiles({
    required Future<List<FileModel>> Function({
      required BuildContext context,
      required List<UploadFileDataModel> files,
    })
        uploadFiles,
  }) {
    _uploadFiles = uploadFiles;
  }

  void setGetFiles({
    required Future<String> Function({
      required String path,
    })
        getStorageDownloadURL,
    required Future<Uint8List?> Function({
      required String path,
    })
        getStorageData,
  }) {
    _getStorageDownloadURL = getStorageDownloadURL;
    _getStorageData = getStorageData;
  }

  void setFirebaseKey({
    required String firebaseApiKey,
  }) {
    _firebaseApiKey = firebaseApiKey;
  }

  static const mobileWidth = 750;
  static const mobileHeight = 500;
}
