// 🎯 Dart imports:
import "dart:async";

// 🐦 Flutter imports:
import 'package:collection/collection.dart';
import "package:flutter/material.dart";

// 📦 Package imports:
import 'package:shared_preferences/shared_preferences.dart';
import 'package:verttice_ui/verttice_ui.dart';

class ThemedColors {
  final Color textColor;
  final Color accentColor;
  final ThemedColor accentColorName;

  const ThemedColors({
    required this.textColor,
    required this.accentColor,
    required this.accentColorName,
  });
}

enum ThemedColor {
  red,
  orange,
  purple,
  blue,
  teal,
  green,
  cyan,
}

ThemedColors themedColorSwitch(ThemedColor color) {
  switch (color) {
    case ThemedColor.orange:
      return const ThemedColors(
        accentColor: Colors.orange,
        textColor: Colors.white,
        accentColorName: ThemedColor.orange,
      );

    case ThemedColor.cyan:
      return const ThemedColors(
        accentColor: Colors.cyan,
        textColor: Colors.black,
        accentColorName: ThemedColor.cyan,
      );

    case ThemedColor.purple:
      return const ThemedColors(
        accentColor: Colors.purple,
        textColor: Colors.white,
        accentColorName: ThemedColor.purple,
      );
    case ThemedColor.blue:
      return const ThemedColors(
        accentColor: Colors.blue,
        textColor: Colors.black,
        accentColorName: ThemedColor.blue,
      );
    case ThemedColor.teal:
      return const ThemedColors(
        accentColor: Colors.teal,
        textColor: Colors.black,
        accentColorName: ThemedColor.teal,
      );
    case ThemedColor.green:
      return const ThemedColors(
        accentColor: Colors.green,
        textColor: Colors.white,
        accentColorName: ThemedColor.green,
      );

    default:
      return const ThemedColors(
        accentColor: Colors.red,
        textColor: Colors.white,
        accentColorName: ThemedColor.red,
      );
  }
}

final vertticeUIPreferenceProvider =
    StateNotifierProvider<VertticeUIPreferenceNotifier, VertticeUIPreference>(
        (ref) {
  return VertticeUIPreferenceNotifier();
});

@immutable
class VertticeUIPreference {
  final ThemedColors themedColors;
  final bool isDarkTheme;
  final ThemedLocale locale;

  const VertticeUIPreference({
    this.themedColors = const ThemedColors(
      accentColor: Colors.red,
      textColor: Colors.white,
      accentColorName: ThemedColor.red,
    ),
    this.isDarkTheme = false,
    this.locale = ThemedLocale.en,
  });

  VertticeUIPreference copyWith({
    ThemedColors? themedColors,
    bool? isDarkTheme,
    ThemedLocale? locale,
  }) {
    return VertticeUIPreference(
      themedColors: themedColors ?? this.themedColors,
      isDarkTheme: isDarkTheme ?? this.isDarkTheme,
      locale: locale ?? this.locale,
    );
  }
}

class VertticeUIPreferenceNotifier extends StateNotifier<VertticeUIPreference> {
  VertticeUIPreferenceNotifier() : super(const VertticeUIPreference()) {
    _initTheme();
    _initLocale();
  }

  Future _initTheme() async {
    final pref = await SharedPreferences.getInstance();
    final dark = pref.getBool("dark") ?? false;
    String? themedColors = pref.getString("themedColors");
    ThemedColor? colorToSet = ThemedColor.values
        .firstWhereOrNull((element) => element.name == themedColors);

    await setThemedColors(
      VertticeUI.instance.themedColors.forceInitColor
          ? VertticeUI.instance.themedColors.themedColors
          : (colorToSet ?? VertticeUI.instance.themedColors.themedColors),
    );
    await setBrightnessTheme(dark);
  }

  Future<void> setThemedColors(ThemedColor color) async {
    final pref = await SharedPreferences.getInstance();
    await pref.setString("themedColors", color.name);
    themedColorSwitch(color);

    state = state.copyWith(
      themedColors: themedColorSwitch(color),
    );
  }

  Future<void> setBrightnessTheme(bool isDark) async {
    final pref = await SharedPreferences.getInstance();
    await pref.setBool("dark", isDark);
    state = state.copyWith(
      isDarkTheme: isDark,
    );
  }

  void setLocale(ThemedLocale locale) {
    state = state.copyWith(
      locale: locale,
    );

    VertticeUI.instance.localeManager.initLocale(locale);
  }

  void _initLocale() {
    state = state.copyWith(
      locale: VertticeUI.instance.localeManager.locale,
    );
  }
}
