import 'dart:math';

import 'package:flutter/foundation.dart';
import 'package:flutter/material.dart';
import 'package:flutter_markdown/flutter_markdown.dart';
import 'package:html2md/html2md.dart' as html2md;
import 'package:verttice_ui/verttice_ui.dart';

bool _popUpIsOpen = false;

class CookiesConsent {
  final String title;
  final String subtitleHtml;
  final String? privacyPolicyHtml;
  final String acceptLabel;
  final bool alreadyRequestCookiesConsent;
  final CookieLabel necessary;
  final CookieLabel tracking;
  final CookieLabel analytics;
  final CookieLabel marketing;
  final bool showTracking;
  final bool showAnalytics;
  final bool showMarketing;
  CookiesConsent({
    this.privacyPolicyHtml,
    this.alreadyRequestCookiesConsent = false,
    this.title = 'GDPR Notice',
    this.subtitleHtml =
        'We use cookies to offer a better browsing experience, analyze site traffic, personalize content, and serve targeted advertisements. By clicking accept, you consent to our privacy policy & use of cookies.',
    this.acceptLabel = 'Accept',
    this.necessary = const CookieLabel(
      label: "Necessary cookies",
      description: "Used for cookie control. Can't be turned off.",
      allow: true,
    ),
    this.showAnalytics = true,
    this.analytics = const CookieLabel(
      label: "Analytics cookies",
      description:
          "Used to control Google Analytics, a 3rd party tool offered by Google to track user behavior.",
      allow: false,
    ),
    this.showMarketing = true,
    this.marketing = const CookieLabel(
      label: "Marketing cookies",
      description: "Used for marketing data.",
      allow: false,
    ),
    this.showTracking = true,
    this.tracking = const CookieLabel(
      label: "Tracking cookies",
      description: "Used for advertising purposes.",
      allow: false,
    ),
  });

  CookiesConsent copyWith({
    String? title,
    String? subtitleHtml,
    String? privacyPolicyHtml,
    String? acceptLabel,
    bool? alreadyRequestCookiesConsent,
    CookieLabel? necessary,
    CookieLabel? tracking,
    CookieLabel? analytics,
    CookieLabel? marketing,
    bool? showTracking,
    bool? showAnalytics,
    bool? showMarketing,
  }) {
    return CookiesConsent(
      title: title ?? this.title,
      subtitleHtml: subtitleHtml ?? this.subtitleHtml,
      privacyPolicyHtml: privacyPolicyHtml ?? this.privacyPolicyHtml,
      acceptLabel: acceptLabel ?? this.acceptLabel,
      alreadyRequestCookiesConsent:
          alreadyRequestCookiesConsent ?? this.alreadyRequestCookiesConsent,
      necessary: necessary ?? this.necessary,
      tracking: tracking ?? this.tracking,
      analytics: analytics ?? this.analytics,
      marketing: marketing ?? this.marketing,
      showTracking: showTracking ?? this.showTracking,
      showAnalytics: showAnalytics ?? this.showAnalytics,
      showMarketing: showMarketing ?? this.showMarketing,
    );
  }
}

class CookieLabel {
  final String label;
  final String description;
  final bool allow;

  const CookieLabel({
    required this.label,
    required this.description,
    required this.allow,
  });

  CookieLabel copyWith({
    String? label,
    String? description,
    bool? allow,
  }) {
    return CookieLabel(
      label: label ?? this.label,
      description: description ?? this.description,
      allow: allow ?? this.allow,
    );
  }
}

class CookiesConsentIconButton extends StatelessWidget {
  const CookiesConsentIconButton({Key? key}) : super(key: key);

  @override
  Widget build(BuildContext context) {
    if (!kIsWeb) {
      return const SizedBox();
    }
    return ThemedIconButton(
      onPressed: () async {
        await showCookiesConsent(context: context);
      },
      icon: ThemedIcons.cookie(),
    );
  }
}

Future<void> showCookiesConsentOnAppLaunch({
  required BuildContext context,
}) async {
  if (!ThemedPlatform.isWebPlatform) {
    return;
  }

  if (VertticeUI.instance.cookiesConsent.alreadyRequestCookiesConsent) {
    return;
  }
  await showCookiesConsent(context: context);
}

Future<void> showCookiesConsent({
  required BuildContext context,
}) async {
  final vertticeUIConfig = VertticeUI.instance;

  if (_popUpIsOpen) {
    return;
  }
  _popUpIsOpen = true;
  double height = 310;

  if (vertticeUIConfig.cookiesConsent.showTracking) {
    height += 60;
  }
  if (vertticeUIConfig.cookiesConsent.showMarketing) {
    height += 60;
  }
  if (vertticeUIConfig.cookiesConsent.showAnalytics) {
    height += 60;
  }

  await showDialog(
    context: context,
    barrierDismissible: false,
    builder: (BuildContext context) {
      return AlertDialog(
        shape: const RoundedRectangleBorder(
          borderRadius: BorderRadius.all(
            Radius.circular(20.0),
          ),
        ),
        insetPadding: const EdgeInsets.all(15),
        contentPadding: const EdgeInsets.all(15),
        content: SizedBox(
          height: min(height, MediaQuery.of(context).size.height - 30),
          width: min(400, MediaQuery.of(context).size.width - 30),
          child: const Scaffold(
            body: _CookiesConsentView(),
          ),
        ),
      );
    },
  );
  _popUpIsOpen = false;
}

class _CookiesConsentView extends StatefulWidget {
  const _CookiesConsentView({Key? key}) : super(key: key);

  @override
  State<_CookiesConsentView> createState() => __CookiesConsentViewState();
}

class __CookiesConsentViewState extends State<_CookiesConsentView> {
  final vertticeUIConfig = VertticeUI.instance;
  late bool tracking;
  late bool marketing;
  late bool analytics;
  @override
  void initState() {
    super.initState();
    tracking = vertticeUIConfig.cookiesConsent.tracking.allow;
    marketing = vertticeUIConfig.cookiesConsent.marketing.allow;
    analytics = vertticeUIConfig.cookiesConsent.analytics.allow;
  }

  @override
  Widget build(BuildContext context) {
    return Scrollbar(
      thumbVisibility: true,
      child: ListView(
        children: [
          sizedBoxH16,
          Text(
            vertticeUIConfig.cookiesConsent.title,
            style: const TextStyle(
              fontSize: 25,
              color: Colors.red,
            ),
          ),
          sizedBoxH16,
          Text(
            vertticeUIConfig.cookiesConsent.subtitleHtml,
          ),
          sizedBoxH16,
          if (vertticeUIConfig.cookiesConsent.privacyPolicyHtml != null)
            TextButton(
              child: const Text('Please review our privacy policy'),
              onPressed: () async {
                await fullPageDialog(
                  context: context,
                  builder: (BuildContext context) {
                    return Scaffold(
                      appBar: AppBar(
                        title: const Text('Privacy Policy'),
                      ),
                      body: SingleChildScrollView(
                        child: MarkdownBody(
                          data: html2md.convert(vertticeUIConfig
                              .cookiesConsent.privacyPolicyHtml!),
                          selectable: true,
                          onTapLink:
                              (String text, String? href, String title) async {
                            if (await canLaunchUrl(Uri.parse(href ?? ""))) {
                              themedLaunchUrl(Uri.parse(href!));
                            }
                          },
                        ),
                      ),
                    );
                  },
                );
              },
            ),
          sizedBoxH16,
          IgnorePointer(
            ignoring: true,
            child: CheckboxListTile(
              activeColor: Colors.grey,
              value: true,
              title: Text(
                vertticeUIConfig.cookiesConsent.necessary.label,
              ),
              subtitle: Text(
                vertticeUIConfig.cookiesConsent.necessary.description,
              ),
              onChanged: (bool? allow) {},
            ),
          ),
          if (vertticeUIConfig.cookiesConsent.showTracking)
            CheckboxListTile(
              value: tracking,
              title: Text(
                vertticeUIConfig.cookiesConsent.tracking.label,
              ),
              subtitle: Text(
                vertticeUIConfig.cookiesConsent.tracking.description,
              ),
              onChanged: (bool? allow) {
                setState(() {
                  tracking = allow ?? false;
                });
              },
            ),
          if (vertticeUIConfig.cookiesConsent.showMarketing)
            CheckboxListTile(
              value: marketing,
              title: Text(
                vertticeUIConfig.cookiesConsent.marketing.label,
              ),
              subtitle: Text(
                vertticeUIConfig.cookiesConsent.marketing.description,
              ),
              onChanged: (bool? allow) {
                setState(() {
                  marketing = allow ?? false;
                });
              },
            ),
          if (vertticeUIConfig.cookiesConsent.showAnalytics)
            CheckboxListTile(
              value: analytics,
              title: Text(
                vertticeUIConfig.cookiesConsent.analytics.label,
              ),
              subtitle: Text(
                vertticeUIConfig.cookiesConsent.analytics.description,
              ),
              onChanged: (bool? allow) {
                setState(() {
                  analytics = allow ?? false;
                });
              },
            ),
          sizedBoxH16,
          Row(
            mainAxisAlignment: MainAxisAlignment.end,
            crossAxisAlignment: CrossAxisAlignment.center,
            children: [
              SizedBox(
                height: 60,
                width: 100,
                child: TextButton(
                  child: Text(
                    vertticeUIConfig.cookiesConsent.acceptLabel,
                  ),
                  onPressed: () async {
                    await vertticeUIConfig.setCookieConsent(
                      vertticeUIConfig.cookiesConsent.copyWith(
                        alreadyRequestCookiesConsent: true,
                        tracking:
                            vertticeUIConfig.cookiesConsent.tracking.copyWith(
                          allow: tracking,
                        ),
                        marketing:
                            vertticeUIConfig.cookiesConsent.marketing.copyWith(
                          allow: marketing,
                        ),
                        analytics:
                            vertticeUIConfig.cookiesConsent.analytics.copyWith(
                          allow: analytics,
                        ),
                      ),
                    );
                    if (mounted) Navigator.pop(context);
                  },
                ),
              ),
              sizedBoxW8,
            ],
          ),
          sizedBoxH4,
        ],
      ),
    );
  }
}
