// 🎯 Dart imports:
import 'dart:convert';
import 'dart:typed_data';

// 🐦 Flutter imports:
import "package:flutter/material.dart";
import "package:flutter/scheduler.dart";

// 📦 Package imports:
import "package:collection/collection.dart";
import 'package:syncfusion_flutter_datagrid_export/export.dart';
import 'package:syncfusion_flutter_pdf/pdf.dart';
import 'package:syncfusion_flutter_xlsio/xlsio.dart' hide Column, Row;
import 'package:verttice_types/verttice_types.dart';
import 'package:verttice_ui/verttice_ui.dart';

import 'table_content.dart';

class TableFrame extends StatefulWidget {
  final bool enableCopyButton;
  final void Function(List<Map<String, dynamic>>)? selectedListCallback;
  final List<FormItem> columns;
  final List<Map<String, dynamic>> data;
  final double frameWidth;
  final double frameHeight;
  final void Function(
    Map<String, dynamic>? rowData,
    int rowIndex,
  ) handleSelectedRow;
  final List<Widget>? tableActions;
  final String title;
  final String uniqueIdField;
  final double actionsBarHeight;

  const TableFrame({
    Key? key,
    required this.enableCopyButton,
    required this.selectedListCallback,
    required this.columns,
    required this.data,
    required this.frameWidth,
    required this.frameHeight,
    required this.handleSelectedRow,
    required this.title,
    required this.uniqueIdField,
    required this.actionsBarHeight,
    this.tableActions,
  }) : super(key: key);

  @override
  State<TableFrame> createState() => _TableFrameState();
}

class _TableFrameState extends State<TableFrame> {
  final GlobalKey<TableContentState> _dataTableContentKey =
      GlobalKey<TableContentState>();
  final _debouncer = Debouncer(milliseconds: 500);
  final double _separator = 2;
  late double _actionsBarHeight;
  late List<Map<String, dynamic>> _data;
  late List<Map<String, dynamic>> _filteredData;
  String _search = "";
  Map<String, List<FilterField>> _filters = {};

  @override
  void initState() {
    _actionsBarHeight = widget.actionsBarHeight;
    _data = widget.data;
    _filteredData = _data;
    super.initState();
  }

  @override
  void didUpdateWidget(covariant TableFrame oldWidget) {
    super.didUpdateWidget(oldWidget);
    if (!const DeepCollectionEquality().equals(_data, widget.data)) {
      _data = widget.data;
      _filteredData = _filterResult(_search, _filters, _data);
    }
  }

  @override
  void setState(VoidCallback fn) {
    if (mounted) {
      SchedulerBinding.instance.addPostFrameCallback((_) {
        super.setState(fn);
      });
    }
  }

  List<Map<String, dynamic>> _filterResult(
    String search,
    Map<String, List<FilterField>> filters,
    List<Map<String, dynamic>> data,
  ) {
    final result = _searchResult(search, _data).where((rowData) {
      bool include = true;
      for (var filterItem in _filters.entries) {
        final column = widget.columns.firstWhereOrNull(
            (element) => element.field.name == filterItem.key);

        for (var filter in filterItem.value) {
          dynamic value;
          if (column != null && column.field.dataBuilder != null) {
            value = column.field.dataBuilder!(rowData);
          } else {
            value = rowData[filter.field];
          }

          if (!applyTableFilter(
            filter: filter,
            cellData: value,
          )) {
            include = false;
          }
        }
      }
      return include;
    }).toList();

    return result;
  }

  List<Map<String, dynamic>> _searchResult(
    String search,
    List<Map<String, dynamic>> data,
  ) {
    return search == ""
        ? data
        : data
            .where(
              (rowData) => json.encode(rowData).toLowerCase().contains(
                    search.toLowerCase(),
                  ),
            )
            .toList();
  }

  Future<void> _exportDataGridToExcel() async {
    final Workbook workbook = _dataTableContentKey
        .currentState!.dataGridKey.currentState!
        .exportToExcelWorkbook();

    final List<int> bytes = workbook.saveAsStream();
    workbook.dispose();
    await Download.memory(
      data: Uint8List.fromList(bytes),
      fileType: VFileType.xlsx,
      name: '${widget.title}DataGrid',
    );
  }

  Future<void> _exportDataGridToPdf() async {
    final PdfDocument document = _dataTableContentKey
        .currentState!.dataGridKey.currentState!
        .exportToPdfDocument(fitAllColumnsInOnePage: false);

    final List<int> bytes = await document.save();
    await Download.memory(
      data: Uint8List.fromList(bytes),
      fileType: VFileType.pdf,
      name: '${widget.title}Table',
    );

    document.dispose();
  }

  @override
  Widget build(BuildContext context) {
    return SizedBox(
      width: widget.frameWidth,
      height: widget.frameHeight,
      child: Column(
        crossAxisAlignment: CrossAxisAlignment.start,
        mainAxisAlignment: MainAxisAlignment.start,
        children: [
          SizedBox(
            width: widget.frameWidth,
            height: _actionsBarHeight,
            child: Row(
              crossAxisAlignment: CrossAxisAlignment.start,
              mainAxisAlignment: MainAxisAlignment.spaceBetween,
              children: [
                Container(
                  width:
                      widget.frameWidth / (widget.tableActions != null ? 2 : 1),
                  padding: const EdgeInsets.symmetric(horizontal: 5),
                  child: ListView(
                    controller: ScrollController(),
                    scrollDirection: Axis.horizontal,
                    children: [
                      sizedBoxW16,
                      Center(
                        child: SelectableText(
                          widget.title,
                          style: const TextStyle(
                            fontWeight: FontWeight.bold,
                            fontSize: 20,
                          ),
                        ),
                      ),
                      sizedBoxW32,
                      Tooltip(
                        message: "Clear all filters",
                        child: ThemedIconButton(
                          onPressed: () {
                            setState(() {
                              _filteredData = _data;
                              _filters = {};
                              _search = '';
                            });
                          },
                          icon: ThemedIcons.clear(),
                        ),
                      ),
                      sizedBoxW8,
                      Tooltip(
                        message: 'Export to Pdf',
                        child: ThemedIconButton(
                          onPressed: _exportDataGridToPdf,
                          icon: ThemedIcons.pdfFile(),
                        ),
                      ),
                      sizedBoxW8,
                      Tooltip(
                        message: 'Export to Excel',
                        child: ThemedIconButton(
                          onPressed: _exportDataGridToExcel,
                          icon: ThemedIcons.sheet(),
                        ),
                      ),
                      sizedBoxW8,
                      FittedBox(
                        fit: BoxFit.scaleDown,
                        child: SizedBox(
                          width: 170,
                          child: TableSearchBar(
                            changeSearch: (String search) {
                              _debouncer.run(() {
                                setState(() {
                                  _search = search;
                                });
                              });
                            },
                          ),
                        ),
                      ),
                      sizedBoxW8,
                    ],
                  ),
                ),
                if (widget.tableActions != null)
                  Container(
                    padding: const EdgeInsets.symmetric(horizontal: 5),
                    width: widget.frameWidth / 2,
                    child: FadeEdges(
                      child: ListView(
                        controller: ScrollController(),
                        scrollDirection: Axis.horizontal,
                        children: [
                          sizedBoxW32,
                          ...(widget.tableActions!),
                          sizedBoxW8,
                        ],
                      ),
                    ),
                  ),
              ],
            ),
          ),
          ThemedDivider(height: _separator, width: widget.frameWidth),
          SizedBox(
            height: widget.frameHeight - (_actionsBarHeight + _separator + .5),
            width: widget.frameWidth,
            child: TableContent(
              key: _dataTableContentKey,
              filters: _filters,
              onFilterChanged: (Map<String, List<FilterField>> filters) {
                setState(() {
                  _filters = filters;
                  _filteredData = _filterResult(_search, _filters, _data);
                });
              },
              uniqueIdField: widget.uniqueIdField,
              selectedListCallback: widget.selectedListCallback,
              data: _filteredData,
              columns: widget.columns,
              enableCopyButton: widget.enableCopyButton,
              handleSelectedRow: widget.handleSelectedRow,
            ),
          ),
          const Divider(
            height: .5,
          ),
        ],
      ),
    );
  }
}

class TableSearchBar extends StatelessWidget {
  final Function(String val) changeSearch;
  const TableSearchBar({Key? key, required this.changeSearch})
      : super(key: key);

  @override
  Widget build(BuildContext context) {
    return TextField(
      onChanged: changeSearch,
      decoration: InputDecoration(
        suffixIcon: ThemedIcons.search(),
      ),
    );
  }
}
