// 🎯 Dart imports:
import "dart:async";
import 'dart:math';

// 🐦 Flutter imports:
import 'package:file_picker/file_picker.dart';
import 'package:flutter/foundation.dart';
import "package:flutter/material.dart";

// 📦 Package imports:
import "package:mime/mime.dart";
import 'package:collection/collection.dart';
import 'package:file_selector/file_selector.dart';
import 'package:verttice_logic/verttice_logic.dart';
import 'package:verttice_types/verttice_types.dart';
// 🌎 Project imports:
import 'package:verttice_ui/verttice_ui.dart';
import 'package:image/image.dart';

Uint8List _resizeImageAndConvertToPng(
  Uint8List data,
) {
  const size = 1000;
  try {
    final image = decodeImage(
      data,
    )!;
    final sizeMax = max(image.height, image.width);
    if (sizeMax <= size) {
      return Uint8List.fromList(encodePng(image));
    }

    final resizedImage = copyResize(
      image,
      height:
          image.height > image.width || image.height == image.width ? size : -1,
      width: image.height > image.width ? -1 : size,
    );

    return Uint8List.fromList(encodePng(resizedImage));
  } catch (e) {
    debugPrint(e.toString());
    return data;
  }
}

class UploadFile extends StatelessWidget {
  final Future<void> Function({
    required List<FileModel> files,
  }) filesDataCallback;
  final String path;
  final String? buttonText;
  final bool multi;
  final Widget? altWidget;
  final VoidCallback? onDoneUpload;

  const UploadFile({
    Key? key,
    required this.path,
    required this.filesDataCallback,
    this.buttonText,
    this.multi = false,
    this.altWidget,
    this.onDoneUpload,
  }) : super(key: key);

  @override
  Widget build(BuildContext context) {
    if (altWidget != null) {
      return InkWell(
        child: altWidget,
        onTap: () {
          Navigator.push(
            context,
            MaterialPageRoute(
              builder: (context) => UploadFilesAction(
                path: path,
                filesDataCallback: filesDataCallback,
                multi: multi,
                onDoneUpload: onDoneUpload,
              ),
            ),
          );
        },
      );
    }
    return ThemedTextButton(
      buttonText ?? "Upload Files",
      onPressed: () {
        Navigator.push(
          context,
          MaterialPageRoute(
            builder: (context) => UploadFilesAction(
              path: path,
              filesDataCallback: filesDataCallback,
              multi: multi,
              onDoneUpload: onDoneUpload,
            ),
          ),
        );
      },
    );
  }
}

class UploadFilesAction extends StatefulWidget {
  final Future<void> Function({
    required List<FileModel> files,
  }) filesDataCallback;
  final String path;
  final bool multi;
  final VoidCallback? onDoneUpload;
  final Future<List<FileModel>> Function({
    required BuildContext context,
    required List<UploadFileDataModel> files,
  })? altUploadFilesFun;

  const UploadFilesAction({
    Key? key,
    required this.path,
    required this.filesDataCallback,
    this.altUploadFilesFun,
    this.multi = false,
    this.onDoneUpload,
  }) : super(key: key);

  @override
  State<UploadFilesAction> createState() => _UploadFilesActionState();
}

class _UploadFilesActionState extends State<UploadFilesAction> {
  final fileTypes = [
    "jpg",
    "png",
    "jpeg",
    "giff",
    "gif",
    "ico",
    "pdf",
    // "m1v",
    // "ogg",
    // "webm",
    "mp4",
    "m4a",
    "mov",
    // "m4p",
    // "m4b",
    // "m4r",
    // "m4v",
    // "avi",
    // "asf",
    "wma",
    "wmv",
  ];

  bool _isPickingFiles = false;

  List<UploadFileDataModel>? _files;

  Future<void> _getFiles() async {
    try {
      if (_isPickingFiles) {
        return;
      }

      setState(() {
        _isPickingFiles = true;
      });

      List<UploadFileDataModel>? files;

      if (ThemedPlatform.isAndroidNativeApp) {
        FilePickerResult? result = await FilePicker.platform.pickFiles(
          allowMultiple: widget.multi,
          allowedExtensions: fileTypes,
          withData: true,
          type: FileType.custom,
        );

        if (result?.files != null) {
          files = result!.files.map((e) {
            final String fileID = IDs.l16();
            final file = e.bytes;
            String? mimeType = lookupMimeType(e.name, headerBytes: file) ?? "";
            return UploadFileDataModel(
              file: file,
              fileID: fileID,
              mimeType: mimeType,
              path: "${widget.path}/$fileID",
              name: e.name,
              size: e.size,
            );
          }).toList();
        }
      } else {
        final typeGroup = XTypeGroup(label: 'Files', extensions: fileTypes);

        if (widget.multi) {
          final filesToUpload = await openFiles(
            acceptedTypeGroups: [typeGroup],
          );
          if (filesToUpload.isNotEmpty) {
            files = await Future.wait(filesToUpload.map((e) async {
              final String fileID = IDs.l16();
              final file = await e.readAsBytes();
              String? mimeType =
                  lookupMimeType(e.name, headerBytes: file) ?? "";
              return UploadFileDataModel(
                file: file,
                fileID: fileID,
                mimeType: mimeType,
                path: "${widget.path}/$fileID",
                name: e.name,
                size: await e.length(),
              );
            }));
          }
        } else {
          final e = await openFile(acceptedTypeGroups: [typeGroup]);
          if (e != null) {
            final String fileID = IDs.l16();
            final file = await e.readAsBytes();
            String? mimeType = lookupMimeType(e.name, headerBytes: file) ?? "";
            files = [
              UploadFileDataModel(
                file: file,
                fileID: fileID,
                mimeType: mimeType,
                path: "${widget.path}/$fileID",
                name: e.name,
                size: await e.length(),
              )
            ];
          }
        }
      }

      if (files != null && files.isNotEmpty) {
        setState(() {
          _files = _files == null ? files : [..._files!, ...files!];
        });
      }
    } catch (e) {
      debugPrint(e.toString());
      ThemedToast.error(context: context, message: "Failed to get files");
    } finally {
      setState(() {
        _isPickingFiles = false;
      });
    }
  }

  Future<void> _uploadAndReturnImageAndUrl() async {
    if (_isPickingFiles) {
      return;
    }
    if (_files == null || _files!.isEmpty) {
      return;
    }
    setState(() {
      _isPickingFiles = true;
    });
    try {
      if (_files != null && _files!.length > 20) {
        ThemedToast.error(
          context: context,
          message: "Can't upload more than 20 files at once, please try again",
        );
      } else {
        List<FileModel> allImageRes = [];
        List<UploadFileDataModel> fileToUpload =
            _files!.where((element) => element.file != null).toList();
        if (ThemedPlatform.isAndroidNativeApp) {
          fileToUpload =
              await Future.wait(fileToUpload.mapIndexed((index, file) async {
            if (file.mimeType != null && file.mimeType!.contains('image')) {
              try {
                ThemedToast.info(
                  context: context,
                  message:
                      'Compressing file ${index + 1}/${fileToUpload.length}',
                );
                final compressedFile = (await compute<Uint8List, Uint8List>(
                  _resizeImageAndConvertToPng,
                  file.file!,
                ));

                return file.copyWith(file: compressedFile);
              } catch (e) {
                return file;
              }
            }
            return file;
          }).toList());
        }
        ThemedToast.info(
          context: context,
          message: 'Uploading files',
        );

        if (widget.altUploadFilesFun != null) {
          final response = await widget.altUploadFilesFun!(
            context: context,
            files: fileToUpload,
          );

          if (response.isEmpty) throw "Res err";
          allImageRes = response;
        } else {
          final response = await VertticeUI.instance.uploadFiles(
            context: context,
            files: fileToUpload,
          );
          debugPrint('Finish upload all ${response.length}');

          if (response.isEmpty) {
            throw "Res err";
          }
          allImageRes = response;
        }

        await widget.filesDataCallback(files: allImageRes);
        if (widget.onDoneUpload != null) {
          widget.onDoneUpload!();
        }
      }
    } catch (e) {
      ThemedToast.error(
        context: context,
        message: "Failed to upload files, please try again",
      );
    } finally {
      setState(() {
        _isPickingFiles = false;
        _files = null;
      });
      Navigator.of(context).pop();
    }
  }

  @override
  void initState() {
    super.initState();
  }

  @override
  void dispose() {
    super.dispose();
  }

  @override
  Widget build(BuildContext context) {
    return WillPopScope(
      onWillPop: () async {
        setState(() {
          _isPickingFiles = false;
          _files = null;
        });
        return true;
      },
      child: Scaffold(
        appBar: ThemedAppBar(
          navigateBackButton: true,
          appBarHeight: 55,
          actions: <Widget>[
            sizedBoxW4,
            if (_isPickingFiles) const Center(child: ThemedProgress()),
            sizedBoxW4,
            if (!_isPickingFiles)
              Padding(
                padding: const EdgeInsets.all(10.0),
                child: ThemedTextButton(
                  "Select Files",
                  onPressed: () async {
                    await _getFiles();
                  },
                ),
              ),
            sizedBoxW4,
            if (_files != null && _files!.isNotEmpty && !_isPickingFiles)
              Tooltip(
                message: 'Save Files',
                child: Padding(
                  padding: const EdgeInsets.all(4.0),
                  child: ThemedIconButton(
                    icon: ThemedIcons.check(),
                    onPressed: () async {
                      await _uploadAndReturnImageAndUrl();
                    },
                  ),
                ),
              ),
            sizedBoxW8,
          ],
        ),
        body: _files == null || _files!.isEmpty
            ? const Center(
                child: Padding(
                  padding: EdgeInsets.all(32.0),
                  child: ThemedText(
                    "Click on select files to upload files",
                    showThemed: true,
                  ),
                ),
              )
            : FileListView(
                deleteFileCallback: (FileModel file) {
                  setState(() {
                    _files!.removeAt(int.parse(file.fileID!));
                  });
                },
                fromMemory: true,
                showAsGrid: true,
                files: _files!
                    .mapIndexed(
                      (index, file) => FileModel(
                        file: file.file,
                        fileID: index.toString(),
                        mimeType: file.mimeType,
                      ),
                    )
                    .toList(),
              ),
      ),
    );
  }
}
